@@ -2,7 +2,7 @@ use strict;
 use warnings;
 
 package IPC::Open3::Callback::Command;
-$IPC::Open3::Callback::Command::VERSION = '1.13';
+$IPC::Open3::Callback::Command::VERSION = '1.14';
 # ABSTRACT: A utility class that provides subroutines for building shell command strings.
 
 use Exporter qw(import);
@@ -90,16 +90,39 @@ sub cp_command {
         }
         else {
             # default, use tar
+            my @parts             = ("tar c -C $source_path .");
+            my @destination_parts = ();
+            if ( $cp_options{status} ) {
+                push(
+                    @parts,
+                    join(
+                        '',
+                        'pv -f -s `',
+                        _sudo_command(
+                            $source_command_options
+                                && $source_command_options->get_sudo_username(),
+                            pipe_command( "du -sb $source_path", 'cut -f1' )
+                        ),
+                        '`'
+                    )
+                );
+            }
             if ( $cp_options{compress} ) {
-                $source_command = command( "tar cz -C $source_path .", $source_command_options );
-                $destination_command =
-                    command( "tar xz -C $destination_path", $destination_command_options );
-            }
-            else {
-                $source_command = command( "tar c -C $source_path .", $source_command_options );
-                $destination_command =
-                    command( "tar x -C $destination_path", $destination_command_options );
+                push( @parts,             'gzip' );
+                push( @destination_parts, 'gunzip' );
             }
+            push(
+                @destination_parts,
+                _sudo_command(
+                    $destination_command_options
+                        && $destination_command_options->get_sudo_username(),
+                    "tar x -C $destination_path"
+                )
+            );
+
+            $source_command = command( pipe_command(@parts), $source_command_options );
+            $destination_command = command( pipe_command(@destination_parts),
+                $destination_command_options->clone( sudo_username => undef ) );
         }
     }
 
@@ -186,6 +209,14 @@ sub sed_command {
     );
 }
 
+sub _sudo_command {
+    my ( $sudo_username, $command ) = @_;
+    if ( defined($sudo_username) ) {
+        $command = "sudo " . ( $sudo_username ? "-u $sudo_username " : '' ) . $command;
+    }
+    return $command;
+}
+
 sub write_command {
     my ( $filename, @lines, $write_options, $command_options );
     $filename        = shift;
@@ -247,9 +278,7 @@ sub wrap {
 
             $command =~ s/^(.*?[^\\]);$/$1/;    # from find -exec
 
-            if ( defined($sudo_username) ) {
-                $command = "sudo " . ( $sudo_username ? "-u $sudo_username " : '' ) . $command;
-            }
+            $command = _sudo_command( $sudo_username, $command );
 
             $destination_command .= $command;
         }
@@ -275,16 +304,36 @@ sub wrap {
 }
 
 package IPC::Open3::Callback::Command::CommandOptions;
-$IPC::Open3::Callback::Command::CommandOptions::VERSION = '1.13';
+$IPC::Open3::Callback::Command::CommandOptions::VERSION = '1.14';
 use parent qw(Class::Accessor);
 __PACKAGE__->follow_best_practice;
-__PACKAGE__->mk_accessors(qw(hostname pretty ssh sudo_username username));
+__PACKAGE__->mk_ro_accessors(qw(hostname pretty ssh sudo_username username));
 
 sub new {
     my ( $class, @args ) = @_;
     return bless( {}, $class )->_init(@args);
 }
 
+sub clone {
+    my ( $instance, %options ) = @_;
+    if ( exists( $instance->{hostname} ) && !exists( $options{hostname} ) ) {
+        $options{hostname} = $instance->{hostname};
+    }
+    if ( exists( $instance->{ssh} ) && !exists( $options{ssh} ) ) {
+        $options{ssh} = $instance->{ssh};
+    }
+    if ( exists( $instance->{username} ) && !exists( $options{username} ) ) {
+        $options{username} = $instance->{username};
+    }
+    if ( exists( $instance->{sudo_username} ) && !exists( $options{sudo_username} ) ) {
+        $options{sudo_username} = $instance->{sudo_username};
+    }
+    if ( exists( $instance->{pretty} ) && !exists( $options{pretty} ) ) {
+        $options{pretty} = $instance->{pretty};
+    }
+    return new( ref($instance), %options );
+}
+
 sub _init {
     my ( $self, %options ) = @_;
 
@@ -309,7 +358,7 @@ IPC::Open3::Callback::Command - A utility class that provides subroutines for bu
 
 =head1 VERSION
 
-version 1.13
+version 1.14
 
 =head1 SYNOPSIS
 
@@ -455,6 +504,13 @@ compression.
 If supplied and true, then this is a file copy, otherwise it is a directory 
 copy.
 
+=item status
+
+If supplied, a status indicator will be printed to STDERR.  This option uses
+the unix C<pv> command which is typically not installed by default.  Also, this
+option only works with C<archive> set to tar (the default), and C<file> set to
+false (the default).
+
 =back
 
 =head2 mkdir_command( $path1, $path2, ..., $pathN, [$command_options] )
